/**
 * @author wangwei
 * Date: 18/5/8
 */
require('../check-versions')();
require('shelljs/global');

var inquirer = require('inquirer');
var chalk = require('chalk');
var columnsComponents = require('../json/columns.json');
var pkgComponents = require('../json/pkg.json');
var replace = require("replace");
var fs = require("fs");
var createUtils = require('../utils/createUtils')
var publishUtils = require('../utils/publishUtils')

let choCreateMode = function () {
  inquirer.prompt([{
    name: 'conform',
    message: `选择需要创建的工程`,
    type: 'list',
    default: 0,
    choices: [{
      name: '创建业务组件',
      value: 1
    }, {
      name: '创建栏目',
      value: 2
    }, {
      name: '创建pc-ui组件',
      value: 3
    }, {
      name: '创建m-ui组件',
      value: 4
    }]
  }]).then(function (answers) {
    var config = null;
    switch(answers.conform){
      case 1:
        config = require('../../packages/config.js');
        break;
      case 2:
        config = require('../../packages_columns/config.js');
        break;
      case 3:
        config = require('../../packages_pc_ui/config.js');
        break;
      case 4:
        config = require('../../packages_m_ui/config.js');
        break;
      default:
        console.log("状态无效！")
      break;
    }
    console.log(config.info);
    if(config)create(config);
  })
}


let create = function (config) {
  inquirer.prompt([
    {
      name: 'name',
      message: '组件名',
      type: 'input',
      default: ''
    }
  ]).then(function (answers) {

    if(!answers.name){
      console.log(`必须入组件名`);
      create(config)
      return;
    }
    answers.name = answers.name.trim();
    answers.name = config.tag+answers.name;

    if(columnsComponents[answers.name] || pkgComponents[answers.name]){
      console.log(`已存在组件[${answers.name}]`);
      create(config)
      return;
    }
    console.log(`创建初始化组件[${answers.name}]`);
    let changeName = createUtils.changeName(answers.name)
    console.log(chalk.green(`------创建模板`));
    let sourceDir = `${config.dir}\\${answers.name}`;
    fs.mkdirSync(sourceDir);
    fs.mkdirSync(`${sourceDir}\\src`);
    fs.mkdirSync(`${sourceDir}\\src\\css`);
    fs.mkdirSync(`${sourceDir}\\src\\img`);
    fs.mkdirSync(`${sourceDir}\\src\\themes`);

    // 拷贝文件列表
    let copyFileList = ['index.js','src\\cap4-button.vue','src\\css\\cap4-button.css']
    copyFileList.forEach(item=>{
      let copyYuan = `packages_examples\\${config.examples}\\${item}`;
      let copyOver = `${config.dir}\\${answers.name}\\${item}`.replace('cap4-button',answers.name);
      console.log(copyOver);
      // 创建读取流
      let readable  = fs.createReadStream( copyYuan );
      // 创建写入流
      let writable  = fs.createWriteStream( copyOver );
      // 通过管道来传输流
      readable.pipe( writable );
    })
    // 拷贝文档
    let copyYuanMd = `packages_examples\\${config.examples}\\cap4-button.md`;
    let copyOverMd = `examples\\${config.docs}\\${answers.name}.md`;

    // 创建读取流
    let readableMd  = fs.createReadStream( copyYuanMd );
    // 创建写入流
    let writableMd  = fs.createWriteStream( copyOverMd );
    // 通过管道来传输流
    readableMd.pipe( writableMd );

    // 复制创建文档
    // let copyYuan = `examples\\docs\\cap4-button.md`;
    // let copyOver = `examples\\docs\\${answers.name}.md`;
    // let readable  = fs.createReadStream( copyYuan );
    // let writable  = fs.createWriteStream( copyOver );
    // readable.pipe( writable );

    // 通过管道来传输流

    // 注册路由 暂无
    setTimeout(()=>{
      replace({
        regex: "cap4-button",
        replacement: answers.name,
        paths: [copyOverMd],
        recursive: true,
        silent: true,
      });
      replace({
        regex: "Cap4Button",
        replacement: changeName,
        paths: [copyOverMd],
        recursive: true,
        silent: true,
      });
      replace({
        regex: "Cap4Button",
        replacement: changeName,
        paths: [`${config.dir}\\${answers.name}`],
        recursive: true,
        silent: true,
      });
      replace({
        regex: "cap4-button",
        replacement: answers.name,
        paths: [`${config.dir}\\${answers.name}`],
        recursive: true,
        silent: true,
      });
    },100)
    // 注册组件
    console.log(chalk.green(`------注册组件`));
    fs.readdir(config.dir, function (err, files) {
      console.log(chalk.green(`------>生成json`));
      var componentsJson = publishUtils.dirToJson(files,config);
      componentsJson[answers.name] = `./${config.dir}/${answers.name}/index.js`;
      fs.writeFileSync(config.componentsPath+config.componentsJson, JSON.stringify(componentsJson));
    })
  })
}

choCreateMode();


