'use strict'
const path = require('path')
const config = require('../config')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const packageConfig = require('../package.json')
var fs = require("fs");

exports.copyFile =  function(path1,path2) {
  // 拷贝文档
  let copyYuanMd = path1;
  let copyOverMd = path2;

  // 创建读取流
  let readableMd  = fs.createReadStream( copyYuanMd );
  // 创建写入流
  let writableMd  = fs.createWriteStream( copyOverMd );
  // 通过管道来传输流
  readableMd.pipe( writableMd );
}

exports.deleteFolder= function(path) {
  exec("rimraf "+path);
};

// process.env.BUILD_PKG_MODE // all sub
// process.env.BUILD_PKG_TYPE // def:业务组件库，columns:栏目组件
exports.getConfigJson = function () {
  var Components = null;
  var str = '';
  if(!process.env.BUILD_PKG_TYPE || process.env.BUILD_PKG_TYPE == 'def') {
    str = 'pkg';
  }
  var config = null;
  switch(process.env.BUILD_PKG_TYPE){
    case 'columns':
      config = require('../packages_columns/config.js');
      break;
    case 'pc_ui':
      config = require('../packages_pc_ui/config.js');
      break;
    case 'm_ui':
      config = require('../packages_m_ui/config.js');
      break;
    case 'def':
    default:
      config = require('../packages/config.js');
      break;
  }
  var Components = null;
  if(process.env.BUILD_PKG_MODE == 'all'){
    Components = require(`./json/${config.componentsJson}`);
  }else{
    Components = require(`./json/${config.componentsSubJson}`);
  }


  return Components;
}


exports.assetsPath = function (_path) {
  const assetsSubDirectory = process.env.NODE_ENV === 'production'
    ? config.build.assetsSubDirectory
    : config.dev.assetsSubDirectory

  return path.posix.join(assetsSubDirectory, _path)
}

exports.cssLoaders = function (options) {
  options = options || {}

  const cssLoader = {
    loader: 'css-loader',
    options: {
      sourceMap: options.sourceMap
    }
  }

  const postcssLoader = {
    loader: 'postcss-loader',
    options: {
      sourceMap: options.sourceMap
    }
  }

  // generate loader string to be used with extract text plugin
  function generateLoaders (loader, loaderOptions) {
    const loaders = options.usePostCSS ? [cssLoader, postcssLoader] : [cssLoader]

    if (loader) {
      loaders.push({
        loader: loader + '-loader',
        options: Object.assign({}, loaderOptions, {
          sourceMap: options.sourceMap
        })
      })
    }

    // Extract CSS when that option is specified
    // (which is the case during production build)
    if (options.extract) {
      return ExtractTextPlugin.extract({
        use: loaders,
        fallback: 'vue-style-loader'
      })
    } else {
      return ['vue-style-loader'].concat(loaders)
    }
  }

  // https://vue-loader.vuejs.org/en/configurations/extract-css.html
  return {
    css: generateLoaders(),
    postcss: generateLoaders(),
    less: generateLoaders('less'),
    sass: generateLoaders('sass', { indentedSyntax: true }),
    scss: generateLoaders('sass'),
    stylus: generateLoaders('stylus'),
    styl: generateLoaders('stylus')
  }
}

// Generate loaders for standalone style files (outside of .vue)
exports.styleLoaders = function (options) {
  const output = []
  const loaders = exports.cssLoaders(options)

  for (const extension in loaders) {
    const loader = loaders[extension]
    output.push({
      test: new RegExp('\\.' + extension + '$'),
      use: loader
    })
  }

  return output
}

exports.createNotifierCallback = () => {
  const notifier = require('node-notifier')

  return (severity, errors) => {
    if (severity !== 'error') return

    const error = errors[0]
    const filename = error.file && error.file.split('!').pop()

    notifier.notify({
      title: packageConfig.name,
      message: severity + ': ' + error.name,
      subtitle: filename || '',
      icon: path.join(__dirname, 'logo.png')
    })
  }
}
/**
 * 增加 hljs 的 classname
 */
exports.wrapCustomClass = function (render) {
  return function (...args) {
    return render(...args)
      .replace('<code class="', '<code class="hljs ')
      .replace('<code>', '<code class="hljs">')
  }
}
var wrap = function(render) {
  return function() {
    return render.apply(this, arguments)
      .replace('<code class="', '<code class="hljs ')
      .replace('<code>', '<code class="hljs">');
  };
};
/**
 * Format HTML string
 */
exports.convertHtml = function (str) {
  return str.replace(/(&#x)(\w{4});/gi, $0 => String.fromCharCode(parseInt(encodeURIComponent($0).replace(/(%26%23x)(\w{4})(%3B)/g, '$2'), 16)))
}
