const path = require('path')
const utils = require('./utils')
const config = require('../config')
const vueLoaderConfig = require('./vue-loader.conf')


const webpack = require('webpack')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')
const ProgressBarPlugin = require('progress-bar-webpack-plugin');

let allTruck = true;
if(process.env.BUILD_PKG_TYPE == 'columns') {
  allTruck = false;
}

let libDir = 'lib';
if(process.env.BUILD_PKG_TYPE == 'columns')libDir = 'lib_columns'
if(process.env.BUILD_PKG_TYPE == 'm_ui')libDir = 'lib_m_ui'
if(process.env.BUILD_PKG_TYPE == 'pc_ui')libDir = 'lib_pc_ui'

function execCmd (cmd) {
  return exec('tree').toString().trim()
}

const env = process.env.NODE_ENV === 'testing'
  ? require('../config/test.env')
  : require('../config/prod.env')
// 单独打版用组件列表

function resolve (dir) {
  return path.join(__dirname, '..', dir)
}



const createLintingRule = () => ({
  test: /\.(js|vue)$/,
  loader: 'eslint-loader',
  enforce: 'pre',
  include: [resolve('examples'), resolve('test')],
  options: {
    formatter: require('eslint-friendly-formatter'),
    emitWarning: !config.dev.showEslintErrorsInOverlay
  }
})

let outputObj = {
  path:  path.resolve(__dirname, `../${libDir}`),
  filename: '[name]/index.js'
}
if(process.env.BUILD_PKG_TYPE == 'columns'){
  // outputObj.libraryTarget= "window";
  // outputObj.library= '[name]';
  outputObj.libraryExport = "default";
  outputObj.libraryTarget = "umd";
}else{
  outputObj.libraryExport = "default";
  outputObj.libraryTarget = "umd";
}

outputObj.libraryTarget= "window";
outputObj.library= '[name]';

console.log('outputObj:['+JSON.stringify(outputObj));
module.exports = {
  output: outputObj,
  resolve: {
    extensions: ['.js', '.vue', '.json'],
    alias: {
      'vue$': 'vue/dist/vue.esm.js',
      '@': resolve('examples'),
      '_pkg': resolve('packages'),
      '_columns': resolve('packages_columns'),
      '_pc_ui': resolve('packages_pc_ui'),
      '_m_ui': resolve('packages_m_ui'),
      '_src': resolve('src'),
      '_utils': resolve('src/utils')
    }
  },
  module: {

    rules: [
      ...(config.dev.useEslint ? [createLintingRule()] : []),
      {
        test: /\.md$/,
        loader: 'vue-markdown-loader'
      },
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: vueLoaderConfig
      },
      {
        test: /\.js$/,
        loader: 'babel-loader',
        include: [resolve('examples'),
          resolve('test'),
          resolve('packages'),
          resolve('packages_columns'),
          resolve('packages_examples'),
          resolve('packages_m_ui'),
          resolve('packages_pc_ui'),
          resolve('src'),
          resolve('node_modules/webpack-dev-server/client')]
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name:'img/[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name:'video/[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10000,
          name: 'font/[name].[ext]'
        }
      }
    ]
  },
  plugins: [
    // http://vuejs.github.io/vue-loader/en/workflow/production.html
    new webpack.DefinePlugin({
      'process.env': env
    }),
    new UglifyJsPlugin({
      uglifyOptions: {
        compress: {
          warnings: false,
          drop_debugger: true,
          drop_console: true
        }
      },
      sourceMap: config.build.productionSourceMap,
      parallel: true
    }),
    // extract css into its own file
    new ExtractTextPlugin({
      filename: '[name]/css/[name].css',
      allChunks: allTruck
    }),
    new ProgressBarPlugin()
  ],
  node: {
    // prevent webpack from injecting useless setImmediate polyfill because Vue
    // source contains it (although only uses it if it's native).
    setImmediate: false,
    // prevent webpack from injecting mocks to Node native modules
    // that does not make sense for the client
    dgram: 'empty',
    fs: 'empty',
    net: 'empty',
    tls: 'empty',
    child_process: 'empty'
  }
};
