# 统计排队组件
----
### 引入
```
import Cap4StatisticsQueue from 'cap4-business/lib/cap4-statistics-queue';
import  'cap4-business/lib/cap4-statistics-queue/css/cap4-statistics-queue.css';
```

### 用法
`cap4-statistics-queue`排队组件,用于带有排队机制的接口。
接口返回类型如下：(必须有`state`字段，统计内容在字段`result`中)

注：新增了对旧有接口的支持，如果类型为普通的统计不带排队字段，直接返回传入的`data`

```
{
     "result" : {},
     "state" : {
         "code" : "101",
         "taskCount":"1000",
         "reqId" : "8330040413334832192"
      }
}
```

排序有三个状态
- `100` 正常返回数据
- `101` 队列中，可以取消
- `102` 队列已满。
### 示例


<script>
  import {data} from './egdata/hasQueueStatisticsData.js'
  export default{
    data () {
       return {
          data101: {
            "result" : {},
            "state" : {
                "code" : "101",
                "taskCount":"1000",
                "reqId" : "8330040413334832192"
              }
          },
          data102: {
              "result" : {},
              "state" : {
                  "code" : "102",
                  "taskCount":"1000",
                  "reqId" : "8330040413334832192"
              }
          }
       }
    },
    mounted() {

    },
    methods: {

    }
  }
</script>

#### pc 101态样式
:注一旦状态变为`100`,表明已经取到数据,内部的作用域中变量`slot.dataInfo`里面会注入统计数据。

<div class="demo-block" style="height:300px">
  <cap4-statistics-queue :data="data101">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
</div>

::: demo
```html
<script>
  export default{
    data () {
       return {
          data101: {
            "result" : {},
            "state" : {
                "code" : "101",
                "taskCount":"1000",
                "reqId" : "8330040413334832192"
              }
          }
       }
    }
  }
</script>

  <cap4-statistics-queue :data="data101">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
```
:::

#### 移动 101态样式

<div class="demo-block" style="height:300px">
  <cap4-statistics-queue :data="data101" :platform="2">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
</div>

::: demo
```html
<script>
  export default{
    data () {
       return {
          data101: {
            "result" : {},
            "state" : {
                "code" : "101",
                "taskCount":"1000",
                "reqId" : "8330040413334832192"
              }
          }
       }
    }
  }
</script>

  <cap4-statistics-queue :data="data101" :platform="2">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
```
:::

#### pc 102态样式

<div class="demo-block" style="height:300px">
  <cap4-statistics-queue :data="data102">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
</div>

::: demo
```html
<script>
  export default{
    data () {
       return {
          data102: {
              "result" : {},
              "state" : {
                  "code" : "102",
                  "taskCount":"1000",
                  "reqId" : "8330040413334832192"
              }
          }
       }
    }
  }
</script>
  <cap4-statistics-queue :data="data102">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
```
:::

#### 移动 102态样式

<div class="demo-block" style="height:300px">
  <cap4-statistics-queue :data="data102" :platform="2">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
</div>

::: demo
```html
<script>
  export default{
    data () {
       return {
          data102: {
              "result" : {},
              "state" : {
                  "code" : "102",
                  "taskCount":"1000",
                  "reqId" : "8330040413334832192"
              }
          }
       }
    }
  }
</script>
  <cap4-statistics-queue :data="data102" :platform="2">
       <template slot-scope="slot">
            {{slot.dataInfo}}
      </template>
  </cap4-statistics-queue>
```
:::

### Attributes
| 参数      | 说明                                 | 类型      | 可选值       | 默认值   |
|---------- |------------------------------------ |---------- |------------- |-------- |
|data      |	由排队接口返回是数据，数据内部包含有【state】排序字段 |	Object   |	—           |	—       |
|platform      |	平台 |	Number   |	1：pc 2：手机           |	1       |

### Event
| 事件名称      | 说明       | 回调参数   |
|------------- |----------- |---------  |
|loop         |后台返回101 排序状态，组件会进入循环状态，取到数据的三秒后再次回调loop方法（新增自动请求，但是必须依赖栏目垫片组件）| state  |
|cancel         |在101排序态下取消排序回调（注：如果为102队列已满，点击确定不再调用后台，所以不会触发此回调）（新增自动请求，但是必须依赖栏目垫片组件）| state  |
