# 组件名
----
`cap4-table`组件调用方法

```js
import { Cap4Table } from 'cap4-business/lib/cap4-table'; // 基础ui表格
import Cap4BusinessTable from 'cap4-business/lib/cap4-table'; // 基于上面表格所封装的业务组件，内置了穿透、查询等逻辑
import 'cap4-business/lib/cap4-table/css/cap4-table.css';

```

### 特别说明
    Cap4Table定位是一个ui组件，其用法如下，基本上不涉及现有业务上的东西；
    Cap4BusinessTable定位是一个业务组件，它是在Cap4Table上进行的无损封装，主要的体现在：
        1.它仅仅是对Cap4Table预置了一些默认行，比如
            a.设置排序时，点击排序会自动去请求数据并替换data;
            b.拖拽完成后，自动保存拖拽信息到本地localStorage;
            c.点击行，可以穿透到对应的页面，前提是设置了可以穿透;
            d.设置了默认渲染函数renderBodyTd;
        2.它是基于现有的业务场景进行的封装，所以使用的时候还得注意外部条件（是否在V5环境等这些）的支持;
        3.它支持所有Cap4Table所支持的Attribute、Event和slot;由于是业务封装，在使用时不再需要传入data和columns，
          只需要传入
            preview（是否可以穿透，Boolean，可选）、
            appId（大家都知道的一个Id，[String, Number]，必选）、
            queryParams（查询条件，Object，必选，支持v-model），
          至于其他Cap4Table支持的参数、事件、slot都不受影响，并且用法相同，唯一需要注意的是，当开发者在该业务组件上
          监听了某一个事件后，该组件默认响应的该事件函数将不会执行（被重写）。
### 基本用法
展示`cap4-table`组件

<div class="demo-block">
  <Cap4Table
  :data='data'
  :columns='columns'
  :loading='loading'
  :highlightRow='true'
  :sortColumn='true'
  @sortChange='sortChange'
  @selectChange='selectChange'
  @currentChange='currentChange'
  @rowClick='rowClick'
  @dragDone='dragDone'
  @></Cap4Table>
</div>
<script>
    export default {
        data(){
            return {
                data : [
                    {
                        name : '张三',
                        age : 24,
                        gender : '男'
                    },
                    {
                        name : '李四',
                        age : 23,
                        gender : '男'
                    },
                    {
                        name : '王五',
                        age : 22,
                        gender : '女'
                    },
                    {
                        name : '赵六',
                        age : 21,
                        gender : '男'
                    }
                ],
                 columns:[
                      {
                         type : 'selection',
                         realWidth : 60
                      },
                      {
                         type : 'index',
                         realWidth : 60
                      },
                     {
                         key : 'name',
                         value : '姓名',
                         fixed : 'left'
                     },
                     {
                         key : 'age',
                         value : '年龄'
                     },
                     {
                         key : 'gender',
                         value : '性别',
                         fixed : 'right'
                     }
                 ],
                 loading : true
            }
        },
        mounted(){
            setTimeout(() => this.loading = false, 1000);
        },
        methods : {
            sortChange(array){
                alert(`您改变了排序规则，当前排序字段为:\n${JSON.stringify(array)}`)
            },
            selectChange(array){
                alert(`您改变了选中列，当前选中字段行的序号为:${array.map(li=>li._index + 1)}`)
            },
            currentChange(li, old){
                alert(`第${li._index + 1}行被高亮显示了`)
            },
            rowClick(li){
                alert(`您点击了第${li.rowIndex + 1}行`)
            },
            dragDone(item, array){
                alert(`拖拽完成，当前拖拽列为:\n${JSON.stringify(item)}，\n所有拖拽信息为:\n${JSON.stringify(array)}`)
            }
        }
    };

</script>

::: demo
```html
  <div class="demo-block">
    <Cap4Table
    :data='data'
    :columns='columns'
    :loading='loading'
    :highlightRow='true'
    :sortColumn='true'
    @sortChange='sortChange'
    @selectChange='selectChange'
    @currentChange='currentChange'
    @rowClick='rowClick'
    @dragDone='dragDone'
    @></Cap4Table>
  </div>
  <script>
      export default {
          data(){
              return {
                  data : [
                      {
                          name : '张三',
                          age : 24,
                          gender : '男'
                      },
                      {
                          name : '李四',
                          age : 23,
                          gender : '男'
                      },
                      {
                          name : '王五',
                          age : 22,
                          gender : '女'
                      },
                      {
                          name : '赵六',
                          age : 21,
                          gender : '男'
                      }
                  ],
                   columns:[
                        {
                           type : 'selection',
                           realWidth : 60
                        },
                        {
                           type : 'index',
                           realWidth : 60
                        },
                       {
                           key : 'name',
                           value : '姓名',
                           fixed : 'left'
                       },
                       {
                           key : 'age',
                           value : '年龄'
                       },
                       {
                           key : 'gender',
                           value : '性别',
                           fixed : 'right'
                       }
                   ],
                   loading : true
              }
          },
          mounted(){
              setTimeout(() => this.loading = false, 1000);
          },
          methods : {
              sortChange(array){
                  alert(`您改变了排序规则，当前排序字段为:\n${JSON.stringify(array)}`)
              },
              selectChange(array){
                  alert(`您改变了选中列，当前选中字段行的序号为:${array.map(li=>li._index + 1)}`)
              },
              currentChange(li, old){
                  alert(`第${li._index + 1}行被高亮显示了`)
              },
              rowClick(li){
                  alert(`您点击了第${li.rowIndex + 1}行`)
              },
              dragDone(item, array){
                  alert(`拖拽完成，当前拖拽列为:\n${JSON.stringify(item)}，\n所有拖拽信息为:\n${JSON.stringify(array)}`)
              }
          }
      };

  </script>

```
:::

### Attributes
| 参数      | 说明                                 | 类型      | 可选值       | 默认值   |
|---------- |------------------------------------ |---------- |------------- |-------- |
|data      |	参数 (必选) |	Array   |	—           |	[]       |
|columns      |	参数 (必选)，格式参考示例（注意特殊属性 type，fixed,render,realWidth对应效果） |	Array   |	—           |	[]       |
|width      |	参数 (可选)，设置table的宽度 |	Number   |	—           |	—       |
|height      |	参数 (可选)，设置table的高度 |	Number   |	—           |	—       |
|loading      |	参数 (可选)，异步加载时有用 |	Boolean   |	—           |	—       |
|highlightRow      |	参数 (可选)，点击行时，是否有选中效果 |	Boolean   |	—           |	—       |
|renderBodyTd      |	参数 (可选)，自定义渲染 td，优先级低于columns元素中的render |	Function   |	—           |	—       |
|sortColumn      |	参数 (可选)，是否启用排序 |	Boolean   |	—           |	—       |

### Event
| 事件名称      | 说明       | 回调参数   |
|------------- |----------- |---------  |
|sortChange         |设置sortColumn为true时，点击表头开始排序的回调，在该事件中改变data| （当前设置了排序列组成的数组）  |
|selectChange         |设置columns中的type为selection时，改变选中状态的事件| （当前选中的行组成的数组，当前行）  |
|currentChange         |设置highlightRow为true时，点击行触发| （当前选中行，上一次点击选中行）  |
|rowClick         |点击行触发点击事件| （一个包含当前第几行、当前行信息、当前列信息的对象）  |
|dragDone         |拖拽列头时会改变列的宽度，并触发该事件| (当前拖拽列，所有被拖拽的列组成的数组)  |

### Slot
 | 名称      | 说明       |
 |------------- |----------- |
 |loading         |自定义loading状态样式效果|
 |no-data         |自定义data为空时样式效果|
### Function
 | 名称      | 说明       |
 |------------- |----------- |
 |clearSelect         |清空当前所有选中行|