'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*eslint-disable*/

// ==========================
// 数据转换
var formatData = function formatData(data) {
  var resData = {};
  if (!data) return resData;
  // 已经整理过的
  var keys = (0, _keys2.default)(data);
  var keystr = keys.toString();
  if (keys.length == 3 && keystr.indexOf('realValue') >= 0 && keystr.indexOf('name') >= 0 && keystr.indexOf('display') >= 0) {
    return data;
  }
  if (data.originalField) {
    resData.realValue = data.realValue ? data.realValue : '';
    resData.name = data.name ? data.name : '';
    resData.display = data.display ? data.display : '';
    return resData;
  } else {
    resData.realValue = data.realValue ? data.realValue : '';
    resData.name = data.name ? data.name : '';
    resData.display = data.display ? data.display : '';
    return resData;
  }
};
// 类型判断
var dataType = function dataType(data) {
  if (!!data) {
    if (data.originalField) {
      if (data.originalField.comType == 'TEXT' || data.originalField.comType == 'TEXTAREA') {
        if (data.originalField.dbType == 'DECIMAL') {
          return 'number';
        } else {
          return 'string';
        }
      } else if (data.originalField.comType == 'DATE' || data.originalField.comType == 'DATETIME') {
        return 'date';
      } else {
        return 'string';
      }
    } else {
      if (typeof data.realValue == 'number') {
        return 'number';
      } else {
        if (data.inputType == 'text' || data.inputType == 'textarea') {
          if (data.fieldType == 'DECIMAL') {
            return 'number';
          } else if (data.inputType == 'date' || data.inputType == 'datetime') {
            return 'date';
          } else {
            return 'string';
          }
        } else {
          return 'string';
        }
      }
    }
  } else {
    return null;
  }
};
// ----------------------------------------------------

function formatDate(date, fmt) {
  if (/(y+)/.test(fmt)) {
    fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
  }
  var o = {
    'M+': date.getMonth() + 1,
    'd+': date.getDate(),
    'h+': date.getHours(),
    'm+': date.getMinutes(),
    's+': date.getSeconds()
  };
  for (var k in o) {
    if (new RegExp('(' + k + ')').test(fmt)) {
      var str = o[k] + '';
      fmt = fmt.replace(RegExp.$1, RegExp.$1.length === 1 ? str : padLeftZero(str));
    }
  }
  return fmt;
};

function padLeftZero(str) {
  return ('00' + str).substr(str.length);
};

// 数字整形
var _nameFormat = function _nameFormat(data) {
  var show = '';
  data = formatData(data);
  if (data && data.name) {
    show = data.name;
  }
  return show;
};
var _titleFormat = function _titleFormat(data1, str) {
  data1 = formatData(data1);
  var show = '';
  if (data1) {
    if (str) {
      show = _nameFormat(data1) + str + formatData(data1).display;
    } else {
      show = formatData(data1).display;
    }
  }
  return show;
};
var _titleFormatByElement = function _titleFormatByElement(key, column, data, str) {
  var name = _elementNameFormat(key, column);
  var data1 = formatData(data[key]);
  var show = '';
  if (data1) {
    if (str) {
      show = name + str + formatData(data1).display;
    } else {
      show = formatData(data1).display;
    }
  }
  return show;
};

// check 百分比 千分位 是否存在
var checkBackFormat = function checkBackFormat(data, num, showUnit) {
  if (!!data.display && data.display.indexOf('%') >= 0) {
    return num + showUnit + '%';
  } else if (!!data.display && data.display.indexOf(',') >= 0) {
    // 整数位置千分位化
    var numstr = num + '';
    if (numstr.indexOf('.') >= 0) {
      var zx = num.split(".");
      var z = (zx[0] || 0).toString().replace(/(\d)(?=(?:\d{3})+$)/g, '$1,');
      var x = zx[1];
      return z + '.' + x + showUnit;
    } else {
      return (num || 0).toString().replace(/(\d)(?=(?:\d{3})+$)/g, '$1,') + showUnit;
    }
  } else {
    return num + showUnit;
  }
};
//判断小数点后有几位 mode：1:强制 2：自动
var numberDecimal = function numberDecimal(money, maxDecimal, mode) {
  var useDecimal = 0;
  var tmpMoney = money.toString();
  var tmpMoneyZ = tmpMoney.indexOf('.');
  if (mode == 1) {
    if (tmpMoneyZ >= 0) {
      var decimal = money.toString().split(".")[1].length;
      if (decimal > maxDecimal) {
        return money.toFixed(maxDecimal);
      } else {
        return money.toFixed(decimal) + getZeroStr(maxDecimal - decimal);
      }
    } else {
      return money + '.' + getZeroStr(maxDecimal);
    }
  }
  if (mode == 2) {
    if (tmpMoneyZ >= 0) {
      var _decimal = money.toString().split(".")[1].length;
      if (_decimal > maxDecimal) {
        useDecimal = maxDecimal;
      } else {
        useDecimal = _decimal;
      }
    }
    var a = money.toFixed(useDecimal);
    return a;
  }
  return money;
};
// 数字整形
var numberFormat = function numberFormat(data, decimal, mode) {
  if (!data) return '';
  if (!data.realValue) {
    if (!!data.display) {
      return data.display;
    } else {
      return '';
    }
  }
  // 后端样式设定了百分比样式%
  if (!!data.display && data.display.indexOf("%") >= 0) {
    return data.display;
  }

  if (!decimal && decimal !== 0) decimal = 1;
  if (!mode) mode = 2;
  var money = 0;
  if (!!data.display && data.display.indexOf('%') >= 0) {
    money = data.realValue * 100;
  } else {
    money = data.realValue;
  }
  //
  if (typeof data.realValue == 'string') {
    data.realValue = parseFloat(data.realValue);
  }
  var showMoney = 0;
  var showUnit = '';
  // 正数的情况
  if (money < 10000 && money >= 0) {
    showMoney = data.realValue;
  }
  if (money >= 10000 && money < 100000000) {
    var tmpMoney = money / 10000;
    showMoney = numberDecimal(tmpMoney, decimal, mode);
    showUnit = '万';
  }
  if (money >= 100000000 && money < 1000000000000) {
    var _tmpMoney = money / 100000000;
    showMoney = numberDecimal(_tmpMoney, decimal, mode);
    showUnit = '亿';
  }
  if (money >= 1000000000000) {
    var _tmpMoney2 = money / 1000000000000;
    showMoney = numberDecimal(_tmpMoney2, decimal, mode);
    showUnit = '兆';
  }
  // 负数的情况
  if (money < 0 && money > -10000) {
    return data.realValue;
  }
  if (money > -100000000 && money <= -10000) {
    var _tmpMoney3 = money / 10000;
    showMoney = numberDecimal(_tmpMoney3, decimal, mode);
    showUnit = '万';
  }
  if (money > -1000000000000 && money <= -100000000) {
    var _tmpMoney4 = money / 100000000;
    showMoney = numberDecimal(_tmpMoney4, decimal, mode);
    showUnit = '亿';
  }
  if (money <= -1000000000000) {
    var _tmpMoney5 = money / 1000000000000;
    showMoney = numberDecimal(_tmpMoney5, decimal, mode);
    showUnit = '兆';
  }
  return checkBackFormat(data, showMoney, showUnit);
};

// 自动
var _autoFormat = function _autoFormat(data, tmpType, param1) {
  var res = '';
  // checkk
  if (!data) return res;
  if (!tmpType) tmpType = 'number';
  // 如果是金额
  if (tmpType == 'money') {
    // 定义默认货币符号
    if (!param1) param1 = '¥';
  }

  // 实际状态
  var type = dataType(data);
  // 数据整理
  data = formatData(data);
  // 有百分号直接返回百分号数字
  if (!!data.display && data.display.indexOf('%') >= 0) {
    return data.display;
  }

  // 自动整理模板类型
  switch (tmpType) {
    case 'number':
      // 数字型
      if (type == 'number') {
        res = numberFormat(data);
      } else {
        res = _titleFormat(data);
      }
      break;
    case 'money':
      // 金额型
      if (type == 'number') {
        var money = numberFormat(data);
        res = money ? param1 + money : '';
      } else {
        var _money = _titleFormat(data);
        res = _money ? _money : '';
      }
      break;
    case 'date':
      if (type == 'number') {
        res = numberFormat(data);
      } else if (type == 'date') {
        if (param1) {
          var tmpDate = new Date(data.realValue);
          // 1）比如模板定义只显示时间；
          if (param1 == 'time') {
            res = formatDate(tmpDate, 'hh:mm');
          } else if (param1 == 'date') {
            // 2）比如模板定义只显示日期；
            res = formatDate(tmpDate, 'yyyy-MM-dd');
          } else if (param1 == 'datetime') {
            // 2）比如模板定义显示日期时间；
            res = formatDate(tmpDate, 'yyyy-MM-dd hh:mm');
          } else if (param1 == 'noToday') {
            //3）比如模板定义为今天不显示，如果是昨天显示为昨天，如果是明天显示为明天，其他时间显示为日期
            var now = new Date();

            if (now.getFullYear() == tmpDate.getFullYear() && now.getMonth() == tmpDate.getMonth() && now.getDate() == tmpDate.getDate()) {
              res = '';
            } else if (now.getFullYear() == tmpDate.getFullYear() && now.getMonth() == tmpDate.getMonth() && now.getDate() - 1 == tmpDate.getDate()) {
              res = '昨天';
            } else if (now.getFullYear() == tmpDate.getFullYear() && now.getMonth() == tmpDate.getMonth() && now.getDate() == tmpDate.getDate() - 1) {
              res = '明天';
            } else {
              res = _titleFormat(data);
            }
          } else {
            // 自定义样式
            res = formatDate(tmpDate, param1);
          }
        } else {
          // 4）模板未定义时按默认样式显示：2017-10-12
          res = _titleFormat(data);
        }
      } else {
        res = _titleFormat(data);
      }
      break;
    default:
      res = _titleFormat(data);
  }
  if (res !== 0 && !res) {
    res = '';
  }
  return res;
};
var _elementNameFormat = function _elementNameFormat(key, column) {
  if (!key || !column || !column.elements || !column.elements.length) {
    return '';
  }
  for (var i = 0; i < column.elements.length; i++) {
    var ele = column.elements[i];
    if (ele.dynamicKey == key) {
      return ele.name;
    }
  }
};
exports.default = {
  methods: {
    autoFormat: function autoFormat(data, tmpType, param1) {
      return _autoFormat(data, tmpType, param1);
    },
    dataFormat: function dataFormat(data) {
      return _autoFormat(data);
    },
    moneyFormat: function moneyFormat(data) {
      return _autoFormat(data, 'money');
    },
    nameFormat: function nameFormat(data) {
      return _nameFormat(data);
    },
    titleFormat: function titleFormat(data1, str) {
      return _titleFormat(data1, str);
    },
    titleFormatByElement: function titleFormatByElement(key, column, data, str) {
      return _titleFormatByElement(key, column, data, str);
    },
    elementNameFormat: function elementNameFormat(key, column) {
      return _elementNameFormat(key, column);
    }
  },
  filters: {
    autoFormat: function autoFormat(data) {
      return _autoFormat(data);
    },
    moneyFormat: function moneyFormat(data) {
      return _autoFormat(data, 'money');
    },
    dataFormat: function dataFormat(data) {
      return _autoFormat(data);
    },
    titleFormat: function titleFormat(data1, str) {
      return _titleFormat(data1, str);
    },
    nameFormat: function nameFormat(data) {
      return _nameFormat(data);
    },
    elementNameFormat: function elementNameFormat(key, column) {
      return _elementNameFormat(key, column);
    }
  }
};