import {pieOption} from './option-pie';
import {radarOption} from './option-radar';
import {lineOption} from './option-line';
import {barOption} from './option-bar';
import deepClone from '_utils/statistics/deepClone';
import * as ctxUtil from './util-ctx';


export const dataToOption = (dataInfo, mode,context)=>{
    let option = {};
    //饼图
    if(mode == 'pie'){
        // check
        if(!dataInfo.info
            || !mode){
            return {};
        }
        option = dataToOptionByPie(dataInfo,context);
    }
    if(mode == 'radar'){
        // check
        if(!dataInfo
            || dataInfo.length <=0
            || !mode){
            return {};
        }
        option = dataToOptionByRadar(dataInfo,context);
    }
    //线图
    if(mode == 'line' || mode == 'bar'){
        // check
        if(!dataInfo
            || !mode){
            return {};
        }
        option = dataToOptionByLine(dataInfo,context);
    }
    return option;
};
// 线图
const dataToOptionByLine = (dataInfos,context)=>{

    let option = null;
    if(context.mode == 'bar'){
        option = deepClone(barOption);
    }else{
        option = deepClone(lineOption);
    }
    // 根据平台修改基础样式 手机
    if(context.platform == 2 && context.platformSub != 'column'){
        // 透明假框
        option.tooltip = {
            trigger: 'axis', //axis
            backgroundColor:'rgba(50,50,50,0.0)',
            borderWidth:1,
            formatter: ' ',
            borderWidth:0,
            borderColor:'#e3e3e3',
            triggerOn:'click',
            textStyle:{color:'#555'}
        };
        option.xAxis.triggerEvent = false;
        option.xAxis.axisPointer = {
            show : true,
            type:'shadow',
            triggerTooltip:true,
            triggerOn:'click'
        };
    }
    // 自定义左宽
    if(context.noData == 1){
        option.grid.x = context.leftWidth - 1;
        option.grid.x2 = 1;
        // 补充差值。底部深度util-line.js中计算获得
        if(context.saveMaxDeep) option.grid.y2=option.grid.y2 + context.saveMaxDeep * 15;
        option.xAxis.axisLabel.show = false;
    }else if(context.noData == 2){
        // option.xAxis.axisLabel.show = true;
        option.grid.x = -1;
    }else{
        option.grid.x = context.leftWidth;
        option.grid.x2 = 1;
        // option.xAxis.axisLabel.show = true;
    }

    // 解析数据
    // xAxis制作 存在第一条数据里
    // 取出第一个数据
    let firstDataInfo = dataInfos[0];
    if(!firstDataInfo || !firstDataInfo.data){
        return {};
    }
    if(context.noData != 1){
        let xAxisData = [];
        for (let j=0;j<firstDataInfo.data.length;j++){
            let dataItem = firstDataInfo.data[j];
            if(dataItem.rowHeaders){
                let lastHeader = dataItem.rowHeaders[dataItem.rowHeaders.length - 1]
                xAxisData.push(lastHeader.lable?lastHeader.lable:'');
            }
        }
        option.xAxis.data = xAxisData;
    }
    //series制作
    let series = [];
    let yMax = 0;
    let yMin = 0;
    let pointCount = 0;
    for (let i=0; i < dataInfos.length;i++) {
        let dataInfo = dataInfos[i];
        let seriesItem = {};
        seriesItem.name = dataInfo.columnName;
        seriesItem.type = context.mode;
        let max = 0;
        let min = 10000000;

        seriesItem.data = dataInfo.data.map(mapItem => {
            pointCount = pointCount + 1;
            if(mapItem.value > max){
                max = mapItem.value;
            }
            if(mapItem.value < min){
                min = mapItem.value;
            }
            if (context.mode == 'bar') {
                // 如果是负数得反向加载圆角
                let dataItem = {value:mapItem.value};
                if(mapItem.value > 0 ){
                    dataItem.itemStyle = {normal: {barBorderRadius : [5, 5, 0, 0]}};
                }else{
                    dataItem.itemStyle = {normal: {barBorderRadius : [0, 0, 5, 5]}};
                }
                return dataItem;
            }else{
                return mapItem.value;
            }
        });
        if(max > yMax){
            yMax = max;
        }
        if(yMin > min){
            yMin = min;
        }
        if(context.noData != 1) {
            // seriesItem.symbol = "none"
            if (context.mode == 'line') {
                if(context.platform == 1){
                    seriesItem.symbolSize = 13;
                    seriesItem.itemStyle = {normal: {color: dataInfo.color, borderColor: dataInfo.color, borderWidth: 4}};
                    seriesItem.lineStyle = {normal: {color: dataInfo.color, width: 4}};
                }else {
                    seriesItem.symbolSize = 8;
                    seriesItem.itemStyle = {normal :{color :dataInfo.color,borderColor :dataInfo.color,borderWidth :3}};
                    seriesItem.lineStyle = {normal :{color :dataInfo.color,width :3}};
                }
                // 计算高宽度
                let paddingw = 5;
                let basew = 7;
                // var planePath = 'path://M940.75037896 581.52285766H1628.55391442a55.61828614 55.61828614 0 0 0 55.61828613-55.61828613V220.00399781a55.61828614 55.61828614 0 0 0-55.61828613-55.61828614H57.33733116a55.61828614 55.61828614 0 0 0-55.61828613 55.61828614v305.90057372a55.61828614 55.61828614 0 0 0 55.61828614 55.61828613h690.75130461c0.08342743 0.37542343 0.18075943 0.76475143 0.30590058 1.14017487l82.76000976 263.8114357a13.90457153 13.90457153 0 0 0 26.52992251 0l82.76000976-263.8114357a13.90457153 13.90457153 0 0 0 0.30590056-1.14017487z';
                //var planePathMin = 'path://M375.534997,115.907183 C375.533716,115.912359 375.532394,115.917526 375.53103,115.922683 L369.625651,138.255395 C369.484465,138.789328 368.937172,139.107712 368.403238,138.966526 C368.058028,138.875243 367.787685,138.606815 367.69395,138.262263 L361.61224,115.907183 L312,115.907183 C309.790861,115.907183 308,114.116322 308,111.907183 L308,90.0383606 C308,87.8292216 309.790861,86.0383606 312,86.0383606 L425,86.0383606 C427.209139,86.0383606 429,87.8292216 429,90.0383606 L429,111.907183 C429,114.116322 427.209139,115.907183 425,115.907183 L375.534997,115.907183 Z'
                // 反
                //var planePathMax = 'path://M446.534997,172.868823 C446.533716,172.873999 446.532394,172.879165 446.53103,172.884323 L440.625651,195.217034 C440.484465,195.750968 439.937172,196.069352 439.403238,195.928165 C439.058028,195.836882 438.787685,195.568455 438.69395,195.223903 L432.61224,172.868823 L383,172.868823 C380.790861,172.868823 379,171.077962 379,168.868823 L379,147 C379,144.790861 380.790861,143 383,143 L496,143 C498.209139,143 500,144.790861 500,147 L500,168.868823 C500,171.077962 498.209139,172.868823 496,172.868823 L446.534997,172.868823 Z';
                var planePathMax ='path://M542.383829,251.890457 L540.456836,254.483473 C540.292125,254.705114 539.978924,254.751264 539.757283,254.586552 C539.719632,254.558572 539.686105,254.525432 539.65769,254.488107 L537.680082,251.890457 L514,251.890457 C512.343146,251.890457 511,250.547311 511,248.890457 L511,242.000217 C511,240.343363 512.343146,239.000217 514,239.000217 L569,239.000217 C570.656854,239.000217 572,240.343363 572,242.000217 L572,248.890457 C572,250.547311 570.656854,251.890457 569,251.890457 L542.383829,251.890457 Z'
                let maxw = max.toString().length * basew + paddingw;
                let minw = min.toString().length * basew + paddingw;
                seriesItem.markPoint = {
                    data: [
                        {
                            type: 'max',
                            name: '最大值',
                            symbolSize: [maxw, 17],
                            symbolOffset: [0, '-80%'],
                            // symbolRotate:0,
                            symbol: planePathMax,
                            label: {
                                normal: {
                                    offset: [0, -1],
                                    color: 'white',
                                    fontSize: 11
                                }
                            },
                            itemStyle: {
                                normal: {
                                    shadowColor: 'rgba(0, 0, 0, 0.2)',
                                    shadowBlur: 10
                                }
                            }
                        },
                        {
                            type: 'min',
                            name: '最小值',
                            symbolSize: [minw, 17],
                            symbolOffset: [0, '-80%'],
                            symbol: planePathMax,
                            label: {
                                normal: {
                                    offset: [0, -1],
                                    color: 'white',
                                    fontSize: 11
                                }
                            },
                            itemStyle: {
                                normal: {
                                    shadowColor: 'rgba(0, 0, 0, 0.2)',
                                    shadowBlur: 10
                                }
                            }
                        }
                    ]
                }
            }
            if (context.mode == 'bar') {
                if(context.computedEchartWidth<context.maxCanvasWidth){
                    if(context.barOption){
                        seriesItem.barWidth = context.barOption.width;
                    }else{
                        if(context.platform == 2){
                            seriesItem.barWidth = 24;
                        }else{
                            seriesItem.barWidth = 34;
                        }
                    }
                }
                seriesItem.barGap = '18%';//柱间距离，可设固定值（如 20）或者百分比（如 '30%'，表示柱子宽度的 30%）。
                seriesItem.itemStyle = {normal: {color: dataInfo.color}};
            }
        }
        option.yAxis.max = yMax;
        // // 防止头溢出
        // if (context.mode == 'line') {
        //     option.grid.y = 40;
        // } else {
        //     option.grid.y = 20;
        // }
        //如果大于400个点，则关闭动画特效
        if(pointCount> 400 || context.print == '1') option.animation=false;
        series.push(seriesItem);
    }
    // 轴计算
    if(context.noData == 2) {
        // option.xAxis.show = false;
        option.series = series;
    }else if (context.noData == 1){
        option.yAxis.show = true;
        option.xAxis.show = false;
        option.series = [{type:'line',data:[yMax,yMin]}];
    }else{
        option.xAxis.show = true;
        option.yAxis.show = true;
        option.series = series;
    }
    //  计算左边宽度
    if(context.noData == 2 || context.noData == 0){
        let yStr = '';
        let yMaxStr = yMax+'';
        let yMinStr = yMin+'';
        let yMaxLen = yMaxStr.length;
        let yMinLen = yMinStr.length;
        // 数字会带千分位，补齐千分位
        if(typeof yMax == 'number'){
            yMaxLen = parseInt(yMaxLen * 4/3);
            yMinLen = parseInt(yMinLen * 4/3);
        }
        if(yMaxLen > yMinLen){
            let count = yMaxLen - yMaxStr.length+1;
            yStr = yMaxStr + new Array( count ).join( ',' );
        }else{
            let count = yMinLen - yMinStr.length+1;
            yStr = yMinStr + new Array( count ).join( ',' );
        }

        let cmpLeftWidth = ctxUtil.getCtx().measureText(yStr).width + 25;
        // pc
        if(context.platform == 1){
            if(cmpLeftWidth > context.leftMaxWidth){
                cmpLeftWidth = context.leftMaxWidth
            }
            if(cmpLeftWidth < context.leftMinWidth){
                cmpLeftWidth = context.leftMinWidth
            }
        }
        // phone
        if(context.platform == 2){
            if(cmpLeftWidth > context.leftMaxWidth){
                cmpLeftWidth = context.leftMaxWidth
            }
            if(cmpLeftWidth < context.leftMinWidth){
                cmpLeftWidth = context.leftMinWidth
            }
        }
        // 最低限锁 40
        if(cmpLeftWidth < 40) cmpLeftWidth = 40;
        option.leftWidth = cmpLeftWidth;
        if(context.noData == 0){
          option.grid.x = option.leftWidth;
        }
    }
    return option
};

//雷达图生成
const dataToOptionByRadar = (dataInfos,context)=>{
    // 继承基本option
    let option = radarOption;
    //手机剔除
    if(context.platform == 2){
        delete option.tooltip;
    }
    let data = []; // 数据
    let indicator = [];// 轴数据

    let max = 0;
    for (let i=0; i < dataInfos.length;i++){
        let dataInfo = dataInfos[i];
        //数据制作
        let dataItem = {};
        dataItem.name = dataInfo.columnName;
        if(dataInfo.data && dataInfo.data.length>0){
            dataItem.value =  dataInfo.data.map(mapItem => {
                if(mapItem.value > max){
                    max = mapItem.value;
                }
                return mapItem.value
            });
        }else{
            dataItem.value = [];
        }

        dataItem.itemStyle = {};
        dataItem.itemStyle.normal  = {};
        dataItem.itemStyle.normal.color = dataInfo.color;
        data.push(dataItem);
    }
    // 取出第一个数据
    let firstDataInfo = dataInfos[0];
    for (let j=0;j<firstDataInfo.data.length;j++){
        let dataInfodataItem = firstDataInfo.data[j];
        let indicatorItem = {};

        indicatorItem.name = dataInfodataItem.itemKey;
        if(j == 0)indicatorItem.axisLabel = {show: true, textStyle: {fontSize: 18, color: '#333'}};
        indicatorItem.max = max;
        indicator.push(indicatorItem);
    }
    if(context.platform != 2){
        // filter 设置
        option.tooltip.formatter = function (params) {
            let res = `<div style="padding: 2px 2px;line-height: 20px">
                        <div style="padding-bottom: 5px">
                        <div style="display: inline-block;vertical-align: top;margin-top: 5px;margin-right: 4px;width: 13px;height: 10px;background-color: ${params.color}"></div>
                        <span style="font-size: 14px">${params.name}</span></div>`;
            if(params.data && params.data.value){
                for (let m=0;m<params.data.value.length;m++){
                    let item = params.data.value[m];
                    let ind = indicator[m];
                    res = res +'<div style="display: inline-block;text-overflow:ellipsis;white-space: nowrap;max-width: 250px;overflow: hidden;padding-left: 20px">'+ ind.name+':</div>' +
                        '<div style="display: inline-block;overflow: hidden;color: #ff8a00">' + item +'</div><br/>';
                }
            }
            return res + '</div>';
        }
    }
    // 计算nameFilter
    // 计算空白距离

    let diameter = context.bodyWidth>context.bodyHeight?context.bodyHeight*0.75 : context.bodyWidth*0.75;
    // 最小边距
    let minPadding =  (context.bodyWidth - diameter)/2 - 35;
    // 最小边距所能显示字数
    let ctx = ctxUtil.getCtx();
    option.radar.name.formatter = function (value, indicator) {
        // value='傻傻的了房傻傻的了房间里水电傻傻的了房间里水电间里水电';
        let valueWidth = ctx.measureText(value).width;
        if(minPadding>valueWidth){
            return value;
        }else{
            value = ctxUtil.ctxGetStrByWidth(value,minPadding);
        }
        // let value = value
        return value;
    }
    // 设置
    option.radar.indicator = indicator;
    option.series[0].data = data;
    return option;
}
//饼图生成方法
const dataToOptionByPie = ({data,info},context)=>{
    // 继承基本option
    let option = deepClone(pieOption);
    // debugger;
    // check
    let sumValue = info.sum ? info.sum : 0;
    // data = data.map((item)=>{
    //     return item.value;
    // });
    // data = [15, 3, 3, 2, 2, 2, 1, 18];
    // data.forEach(v => {
    //    console.log(v/46);
    // });
    // debugger
    if(!data || !data.length){
        let nullData = [];
        let nullItem = {};
        nullItem.itemStyle = {normal:{color:"#cccccc"}};
        nullItem.value = 1;
        nullData.push(nullItem);
        option.series[0].data = nullData;
        option.series[0].label.normal.show = false;
        option.series[0].itemStyle.normal.borderWidth = 0;
    }else{
        option.series[0].data = data;
        option.series[0].label.normal.show = true;
        if(data.length <= 1){
            option.series[0].itemStyle.normal.borderWidth = 0;
        }else{
            option.series[0].itemStyle.normal.borderWidth = 5;
        }
    }
    if(sumValue > 0){
        //百分比规则
        let filter = (params)=>{
            // let parent = parseInt(params.value/sumValue*1000);
            // if(parent<50){
            //     return '';
            // }
            // const z = parseInt(parent/10);
            // const x = (parent%10);
            // let res = z;
            // if(x>0)res= res +'.'+ x;
            if(params.data.realPercent >= 5){
              return params.data.percent;

            }else{
              return '';
            }
        }
        option.series[0].label.normal.formatter = filter;
    }
    // 皮肤处理
    switch(context.$themes){
      case 'black':
        option.series[0].itemStyle.normal.borderColor = '#6c6b6b';
        break;
      default:
    }
    return option;
};

