
import vCheckbox from '../components/vCheckbox.vue'
import vPolygon from '../components/vPolygon.vue'
import vTd from '../components/vTd.vue'
import { getParent } from './utils'

// table-head
export const tableHeadMixin = {
    props : {
        data : {
            type : Array,
            default : () => []
        }
    },
    methods : {
        // 排序
        sort(item){
            if(this.$parent.sortColumn){
                const v = item.orderBy;
                item.orderBy = v === undefined ? 'asc' : v === 'asc' ? 'desc' : undefined;
                this.sortChange(item);
            }
        },
        // 全选、全不选
        check(value){
            const $parent = this.$parent;
            $parent.tableData.forEach(li => li._checked = value);
            $parent.checkedLength = value ? $parent.tableData.length : 0;
            $parent.$emit('selectChange',value ? $parent.tableData : [])
        },
        // 点击了排序
        sortChange({ orderBy, key }){
            const array = this.$parent.sortArray;
            for(let i = 0,len = array.length;i < len; i++){
                if(array[i].key === key){
                    array.splice(i,1);
                    break;
                }
            }
            if(orderBy)array.unshift({ key, orderBy});
            this.$parent.$emit('sortChange',array)
        },
    },
    components : {
        vPolygon,
        vCheckbox
    },
}
export const tableBodyMixin = {
    props : {
        data : {
            type : Array,
            default : () => []
        },
        columns : {
            type : Array,
            required : true,
            default : () => []
        },
        tableId : String
    },
    methods : {
        // 这块处理hover - tr状态
        tableBodyMouseMove(e){
            if(this.$parent.readyDrag)return;
            const tr = getParent(e.target, 'TR');
            if(tr) this.$parent.hoverIndex = +tr.getAttribute('data-index');
        },
        scroll(){
            if(!this.canScroll)return;
            const { $parent, $el : { scrollLeft, scrollTop } } = this;
            $parent.cap4TableHeadStyle.marginLeft = `${-scrollLeft}px`;
            // $parent.fixedRightTableBody.scrollTo(0, scrollTop);
            $parent.fixedLeftTableBody.scrollTop = scrollTop;
            $parent.fixedRightTableBody.scrollTop = scrollTop;
            $parent.fixedRightTableBody.scrollTop = scrollTop;
            $parent.cap4TableBody.scrollTop = scrollTop;
            $parent.cap4TableBody.scrollLeft = scrollLeft;
        },
        // 点击行，会触发rowClick，假如有高亮显示当前行，则会额外触发currentChange
        rowClick($event){
            const tr = getParent($event.target, 'TR');
            if(tr){
                const rowIndex = +tr.getAttribute('data-index');
                const parent = this.$parent;
                const oldRowIndex = parent.oldRowIndex;
                // if(oldRowIndex === rowIndex)return;
                parent.oldRowIndex = rowIndex;
                const currentRow = this.data[rowIndex];
                if(parent.highlightRow){
                    parent.$emit('currentChange',currentRow, this.data[oldRowIndex]);
                }
                const td = getParent($event.target, 'TD');
                if(td){
                    const columnIndex = td.getAttribute('data-index');
                    parent.$emit('rowClick',{
                        rowIndex,
                        row : currentRow,
                        column : this.columns[columnIndex]
                    },$event);
                }
            }
        }
    },
    components : { vTd },
    watch : {
        '$parent.useCssHover' : {
            handler(flag){
                this.$nextTick(()=>{
                    // 使用css展示hover效果
                    if(flag){
                        this.$el.className += ' has-hover';
                    }
                    // 使用js展示hover效果
                    else {
                        this.$el.removeEventListener('mousemove', this.tableBodyMouseMove);
                        this.$el.addEventListener('mousemove', this.tableBodyMouseMove);
                    }
                });
            },
            immediate : true
        }
    },
    mounted(){
        this.$parent[`${this.tableId}`] = document.getElementById(this.tableId);
    },
    beforeDestroy(){
        this.$el.removeEventListener('mousemove', this.tableBodyMouseMove)
    }
}