/**
 * 该文件主要实现drag拖拽功能，兼容处理hover效果
 * */

export const cap4TableDragMixin = {
    data(){
        return {
            readyDrag : false, // 是否准备拖拽
            dragLineStyle : {}, // 拖拽线的位置
            dragingItem : null, // 当前正在拖拽的item
            minWidth : 60, // 最小宽度
            startLocation : undefined, // 开始拖拽的位置
        }
    },
    methods : {
        // 处理鼠标离开table时，hover效果消失
        mouseLeaveTable(){
            this.hoverIndex = -1;
        },
        cursorMouseMove(e){
            // 如果准备拖拽
            if(this.readyDrag){
                const { dragingItem, dragLineStyle, fixedTotalWidth, startLocation, $el, minWidth, scrollWidth } = this;
                const endLocation = e.clientX - $el.getBoundingClientRect().left;
                if(dragingItem.fixed && fixedTotalWidth + endLocation - startLocation > $el.clientWidth - scrollWidth) return;
                if(this.startLocation - endLocation < dragingItem.width - minWidth)dragLineStyle.left = `${endLocation}px`;
            }
        },
        dragEnd(){
            if(this.readyDrag){
                const { dragLineStyle, dragingItem } = this;
                const diff = dragLineStyle.left.slice(0,-2) - this.startLocation;
                dragingItem.width += diff;
                dragingItem.realWidth = dragingItem.width;
                this.realTableWidth += diff;
                this.tableBodyScrollHidden = false; // 去掉overflowX : hidden;

                // 重置状态
                console.log('dragingItem:', dragingItem.realWidth, dragingItem.width);
                this.readyDrag = false;
                dragLineStyle.display = 'none';
                this.$emit('dragDone',dragingItem,this.tableColumns);
                this.dragingItem = null;
                this.$nextTick(()=>{
                    const { offsetHeight, clientHeight, scrollLeft } = this.cap4TableBody;
                    this.scrollWidth2 = offsetHeight - clientHeight;
                    this.cap4TableHeadStyle.marginLeft = `${-scrollLeft}px`;
                });
                this.$el.removeEventListener('mousemove',this.cursorMouseMove);
            }
        }
    }
};
export const tableHeadDragMixin = {
  methods : {
      // 拖拽
      startDrag(e, item){
          const parent = this.$parent;
          const clientHeight = parent.$refs.cap4TableBody.$el.clientHeight;
          const left = e.clientX - parent.$el.getBoundingClientRect().left;
          parent.dragLineStyle = {
              height : `${clientHeight}px`,
              left : `${left}px`,
              display : `block`,
              bottom : `${parent.tableBodyHeight - clientHeight}px`
          };
          parent.readyDrag = true; // 准备开始拖拽
          parent.dragingItem = item;
          parent.startLocation = left;
          // fixed固定左右侧表的宽度总和
          parent.fixedTotalWidth = parent.getWidth(parent.fixedLeftColumns) + parent.getWidth(parent.fixedRightColumns);
          parent.$el.addEventListener('mousemove',parent.cursorMouseMove);
      },
  }
};