/*eslint-disable*/
import {colors} from './constant.js';
import {formatThousand,moneyDecimal} from './moneyFormat.js';
// title设置 屏蔽名单
export const configureSpecialUrl  =  [
    {urlFileds : 'main.do'},
    {urlFileds : 'business.do'},
    {urlFileds : 'businessTemplateController.do'},
    {urlFileds : 'vReport.do'}];

// dataInfo 统计的数据包，根据 selectGroup,selectIndex提取数据
// selectGroup 交叉表的交叉表头
// selectIndex 选择的列
// option:
// option.max Number 最大行，默认为-1，不限制
// option.hansLastData  Boolean 是否存在合计行，默认：true
// option.mode String 根据echart命名，echart类型名 默认：''

let echartDataDefaultOption = {
  max:-1,
  hansLastData:true,
  mode:''
};

export const createEchartData = (dataInfo, selectGroup ,selectIndex,option) => {
  let useDataInfos = [];

  if (!dataInfo || !dataInfo.statisticsInfo) return useDataInfos;
  option = Object.assign(echartDataDefaultOption,option);

  if(option.hansLastData)dataFilter(dataInfo,option);// 剔除计算属性取得lastData

  if(selectIndex>=0){
    const item = dataInfo.statisticsInfo.statsFields[selectIndex];
    option.selectIndex = selectIndex;
    const echartDataInfo = createDataItemToEchartDataInfo(dataInfo,item,selectGroup,option);
    useDataInfos.push(echartDataInfo);
  }else{
    for (let i = 0; i < dataInfo.statisticsInfo.statsFields.length; i++) {
      const item = dataInfo.statisticsInfo.statsFields[i];
      option.selectIndex = i;
      const echartDataInfo = createDataItemToEchartDataInfo(dataInfo,item,selectGroup,option);
      useDataInfos.push(echartDataInfo);
    }
  }
  return useDataInfos;
}

// 解析出带交叉的表头
export const createDataItemToEchartDataInfo = (dataInfo, item, selectGroup,option) => {
  let columnId = '';
  if (selectGroup) {
    columnId = `${selectGroup}${item.aliasTableName}_${item.name}`;
  } else {
    columnId = `${item.aliasTableName}_${item.name}`;
  }
  // 构造echart数据
  const echartDataInfo = dataToEchartData(dataInfo, columnId,option,item);
  echartDataInfo.penetrateEnable = item.penetrateEnable;
  echartDataInfo.statsField = item;
  echartDataInfo.key = columnId;
  echartDataInfo.lastData = dataInfo.statisticsData.lastData;
  echartDataInfo.columnName = item.aliasDisplay;
  return echartDataInfo;
}

// 通用数据解析清单
export const dataToEchartData = (dataInfo, columnId,option,statsField) => {
  let res = {};
  res.value = '';
  let data = dataInfo.statisticsData.data;
  let info = dataInfo.statisticsInfo;
  // 取出合计项
  // (inputData, n, columnId, info, index, mode,statsField)
  // 合计的情况 弹出合计项；
  let topN = getMaxData(dataInfo,option.max,columnId, info, option.selectIndex, option.mode,statsField);

  // 合计值
  let lastData = dataInfo.statisticsData.lastData
  if(lastData && lastData[columnId]){
    let sum = lastData[columnId][1];
    res.info = {sum:sum};
    res.data = setRowHeaderName(topN,info,sum);
  }else{
    res.info = {sum:0};
    res.data = setRowHeaderName(topN,info);
  }
  //  取得轮盘颜色
  let numColors = getColorsByNum( res.data.length);
  for (let i=0;i<res.data.length;i++){
    let item = res.data[i];
    item.capIndex = i;
    item.itemStyle = {normal:{color:numColors[i]}};
  }
  return res;
};

// 日期格式化
export const dateFormat = (data, fmt) => {
    if(!fmt)fmt = "yyyy-MM-dd hh:mm:ss";
    var o = {
        "M+" : data.getMonth()+1,                 //月份
        "d+" : data.getDate(),                    //日
        "h+" : data.getHours(),                   //小时
        "m+" : data.getMinutes(),                 //分
        "s+" : data.getSeconds(),                 //秒
        "q+" : Math.floor((data.getMonth()+3)/3), //季度
        "S"  : data.getMilliseconds()             //毫秒
    };
    if(/(y+)/.test(fmt)) {
        fmt=fmt.replace(RegExp.$1, (data.getFullYear()+"").substr(4 - RegExp.$1.length));
    }
    for(var k in o) {
        if(new RegExp("("+ k +")").test(fmt)){
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length)));
        }
    }
    return fmt;
}
// 剔除计算属性，取得lastData
export const dataFilter = (dataInfo,option) => {
    if(dataInfo.statisticsData.lastData){
        return dataInfo;
    }
    if(dataInfo && dataInfo.statisticsData.data && dataInfo.statisticsData.data.length){
        let lastData = dataInfo.statisticsData.data[dataInfo.statisticsData.data.length - 1];
        if(lastData.rowType === '1'){
            dataInfo.statisticsData.lastData = lastData;
            dataInfo.statisticsData.data = dataInfo.statisticsData.data.filter(item => !item.rowType);
        }else {
            // 自己计算
            let comLastData = {};
            for (let i=0;i<dataInfo.statisticsData.data.length;i++){
                let item = dataInfo.statisticsData.data[i];
                for (let arr in item) {
                    if(item[arr] && typeof item[arr] == 'object'){
                        if(!!comLastData[arr]){
                            comLastData[arr][0] = comLastData[arr][1] = comLastData[arr][1] + valueToNumber(item[arr][1],0);
                        }else{
                            comLastData[arr] = [0,0];
                            comLastData[arr][0] = comLastData[arr][1] = comLastData[arr][1] + valueToNumber(item[arr][1],0);
                        }
                    }
                }
            }
            dataInfo.statisticsData.lastData = comLastData;
        }
    }
    return dataInfo;
}

// 轮盘取色器
export const getColorsByNum = (num)  =>{
    if(num > colors.length){
        let count = Math.ceil(num/colors.length);
        let res = [];
        for (let i=0;i<count;i++){
            res = res.concat( colors );
        }
        return res;
    }else{
        return colors;
    }
}
export const getColorsByIndex = (i)  =>{
    let ci = i%20;
    return colors[ci];
}
export const getColorsByNum2 = (num)  =>{
    if(num >= colors.length){
        return colors;
    }else{
        let offset = colors.length / num;//
        let backI = null;
        let res = [];
        for (let i = 0;i<colors.length;i){
            if(res.length >= num) break;
            const index = parseInt(i);
            if(backI != index){
                res.push(colors[index]);
                backI = index;
            }
            i = i + offset;
        }
        return res;
    }
}
//取得下拉菜单
export const  getSelctList = (dataInfo)  =>{

    // 入口cheeck
    let dataList = [];
    if (!dataInfo || !dataInfo.statisticsInfo){
        return null;
    }
    let colData = dataInfo.statisticsData.colData;
    if(!!colData && colData.length>0){
        let value = '';
        for(let i=0;i<colData.length;i++){
            let selectOption = {};
            selectOption.key = `col_${i}_`;
            selectOption.value = colData[i] ? colData[i][0]:'';
            if(i==0){
                value = selectOption.key;
            }
            dataList.push(selectOption);
        }
        const option = {
            placeholder : '',
            label : 'value',
            value : 'key',
            width : 200
        }
        return {dataList,value,option};
    }else{
        return null;
    }
};
const valueToNumber = (value,def)=>{
    if(def !== 0)def = '';
    let res = 0;
    try {
        var re = /^(\-|\+)?\d+(\.\d+)?$/;
        if (re.test(value)){
            value = parseFloat(value);
            if(typeof value == 'number'){
                res = value;
            }else{
                res = def;
            }
        }
    }catch (e){
        res = def;
    }
    return res;
};
const makeLable = (item,columnId)=>{

};
// 找出最大的n个值，剩余取总// -1：line全部转换启动行头制作，只在第一次循环（index=0）制作
export const getMaxData = (inputData, n, columnId, info, index, mode,statsField) => {
    n = parseInt(n);
    // 排行榜制作
    var topN = [];
    if(!n){
        return topN;
    }

    let dataInfo = {};
    if(inputData.statisticsData && inputData.statisticsInfo){
        dataInfo = inputData;
        inputData = inputData.statisticsData.data
    }
    // 去除计算属性()已经在前文剔除
    // let data = inputData.filter(item => !item.rowType)
    // 字符串补丁总值

    let data = inputData;
    // 启动行头制作
    let topPushItemBackList = [];
    if(n == -1 && index == 0){
        // 制作头游标
        topPushItemBackList.length = info.rowHeaders.length;
    }
    if(n == -1){
        for (let i = 0;i<data.length;i++) {
            let item = data[i];
            let topPushItem = {};
            topPushItem.item = item;
            topPushItem.value = valueToNumber(item[columnId][1]);
            topPushItem.lable = item[columnId][0]?item[columnId][0]:'';
            topPushItem.columnId = columnId;
            //放入队列
            topN.push(topPushItem);
            // 启动行头制作
            if(n == -1 && index == 0){
                topPushItem.rowHeaders =[];
                for (let h =0;h<info.rowHeaders.length;h++){
                    let rheader = info.rowHeaders[h];
                    let key = `${rheader.aliasTableName}_${rheader.name}`;

                    let rowHeaderItem = {};
                    rowHeaderItem.key=key;
                    rowHeaderItem.value=item[key][1];
                    rowHeaderItem.lable=item[key][0];
                    if(h == 0){
                        rowHeaderItem.map = rowHeaderItem.value;
                    }else{
                        rowHeaderItem.map = topPushItem.rowHeaders[h-1].map+','+rowHeaderItem.value;
                    }
                    // 前置比较
                    if(topPushItemBackList[h]){
                        if(topPushItemBackList[h].rowHeaders[h].map === rowHeaderItem.map){
                            rowHeaderItem.colSpan = 0;// 跳过此航头，前置航头+1
                            topPushItemBackList[h].rowHeaders[h].colSpan = topPushItemBackList[h].rowHeaders[h].colSpan + 1;
                        }else{
                            rowHeaderItem.colSpan = 1;
                            topPushItemBackList[h] = topPushItem;
                        }
                    } else {
                        rowHeaderItem.colSpan = 1;
                        topPushItemBackList[h] = topPushItem;
                    }
                    // 加入头
                    topPushItem.rowHeaders.push(rowHeaderItem);
                }

                // 最后缓存
            }
        }
    }else{
        // 垃圾字段回收
        let otherValue = 0;
        // 循环
        for (let i = 0;i<data.length;i++){
            let item = data[i];
            let value = valueToNumber(item[columnId][1]);
            let lable = item[columnId][0]?item[columnId][0]:'';
            if(mode == 'radar'){
                // 雷达图直接放进去，无需排序
                if(topN.length < n){
                    let topPushItem = {};
                    topPushItem.value = value;
                    topPushItem.lable = lable;
                    topPushItem.item = item;
                    topPushItem.columnId = columnId;
                    topN.push(topPushItem);
                }else {
                    break;
                }
            }else{
                //饼图不需要0 的数据
                if(!value)continue;
                //饼图不需要非数字 的数据

                if(topN.length == 0){
                    let topPushItem = {};
                    topPushItem.value = value;
                    topPushItem.lable = lable;
                    topPushItem.item = item;
                    topPushItem.columnId = columnId;
                    topN.push(topPushItem);
                }else{
                    let inFlg = false;
                    for (let j = 0;j<topN.length;j++){
                        let topItem = topN[j];
                        if(value > topItem.value){
                            let topPushItem = {};
                            topPushItem.value = value;
                            topPushItem.lable = lable;
                            topPushItem.item = item;
                            topPushItem.columnId = columnId;
                            //加入排行榜
                            inFlg = true;
                            topN.splice(j,0,topPushItem);
                            // 如果超标了就弹出最后一个
                            if(topN.length > n-1){
                                let overItem = topN.pop();
                                otherValue = otherValue + overItem.value;//替换垃圾
                            }
                            break;
                        }
                        if(j == topN.length-1){
                            if(topN.length < n - 1){
                                //加入排行榜
                                inFlg = true;
                                let topPushItem = {};
                                topPushItem.value = value;
                                topPushItem.lable = lable;
                                topPushItem.item = item;
                                topPushItem.columnId = columnId;
                                topN.push(topPushItem);
                                break;
                            }
                        }
                    }
                    if(!inFlg)otherValue = otherValue + value;
                }
            }
        }
        // 装载其他字段
        if(otherValue>0){
            let topOtherItem = {};
            topOtherItem.value = otherValue;
            topOtherItem.lable = fomatAsServer(otherValue,statsField);
            topOtherItem.itemKey = '其他';
            topOtherItem.name = '其他';
            topN.push(topOtherItem);
        }
    }
    return topN
};
// 跟后端同步转换
const fomatAsServer = (value, statsField) => {
    if(value!==0 && !value)return 0;
    // 小数位
    let digitNum = statsField.digitNum?parseInt(statsField.digitNum):0;
    // has千分位 format_thousand 百分位format_hundred
    // 是否是百分比
    let format_hundred = statsField.formatType?statsField.formatType.indexOf('format_hundred')>=0:false;
    let format_thousand  = statsField.formatType?statsField.formatType.indexOf('format_thousand')>=0:false;

    if(format_hundred){
        value = value * 100;
    }
    if(digitNum > 0){
        value = moneyDecimal(value,digitNum,1);
    }
    if(format_thousand){
        value = formatThousand(value);
    }
    if(format_hundred){
        value = value +'%';
    }
    return value;

}



// 数据namefilter
//行头计算
export const setRowHeaderName = (topN, info, sum) => {
    // 所有百分比的和值
    let allPercent = 0;
    let percentList = [];
    for(let i=0;i<topN.length;i++){
        let topItem = topN[i];
        if(topItem.item){
            // 取得名字
            let name = '';
            let itemKey = '';
            let index = 0;
            info.rowHeaders.forEach(item => {
                let key = `${item.aliasTableName}_${item.name}`;
                let subName = '';
                let subItemKey = '';
                if(topItem.item[key][0] == null){
                    subName = 'null';
                    subItemKey='';
                }else{
                    subName = topItem.item[key][0]?topItem.item[key][0]:'';
                    subItemKey = topItem.item[key][0]?topItem.item[key][0]:'';
                }

                if(index === 0){
                    name = subName;
                    itemKey = subItemKey;
                }else{
                    name = name +'/'+subName;
                    itemKey = itemKey +'/'+subItemKey;
                }
                index = index + 1;
            });
            topItem.name = name;
            topItem.itemKey = itemKey;
        }
        if(sum && sum > 0){
            // 取得百分比
            // let allPercent = 0;
            let sumNumber = valueToNumber(sum);



            // topItem.value = 11;
            if(!!sumNumber && sumNumber>0){
                let parentS = topItem.value / sumNumber;
                let parentF = parseFloat(parentS.toFixed(3));
                allPercent = allPercent + parentF;
                let sortList = {index:i,value:parentF - parentS,parentS:parentS,parentF:parentF};
                 percentList.push(sortList);

                //
                // let parent = Math.floor(topItem.value / sumNumber * 1000);
                // const z = parseInt(parent/10);
                // const x = (parent % 10);
                // parent = z;
                // if(x>0)parent = parent +'.'+ x;
                // topItem.percent = parent + '%';
            }

        }
    }
  if(allPercent > 0){

    // 取得差点 如果大于100%
    if(allPercent > 1){
      // 值降序
      percentList = percentList.sort((a,b)=>{
        return b.value - a.value
      });
      // 取得差多少
      let count = parseInt((allPercent - 1 )/ 0.001);
      for(let p = 0;p<=count-1;p++){
        percentList[p].parentF = percentList[p].parentF - 0.001;
      }
      // index 升序
      percentList = percentList.sort((a,b)=>{
        return  a.index - b.index;
      });
    }
    // 再一次循环
    for(let i=0;i<topN.length;i++) {
      let topItem = topN[i];
        const overPercent = (percentList[i].parentF * 100).toFixed(1);
        topItem.realPercent = parseFloat(overPercent);
        topItem.percent = overPercent + '%';
    }
  }

  return topN;
}

//穿透配置转换
export const penetrationUtil= (item, designId, userConditions, Data, Info )=>{
    // 当前行数据
    const currentRow = item.item;
    // 列表头
    let currentCol = {};
    if (Info) {
        for (let i = 0; i < Info.statsFields.length; i++) {
            const statsField = Info.statsFields[i];
            const diffId = `${statsField.aliasTableName}_${statsField.name}`;
            if (item.columnId.indexOf(diffId)>=0) {
                currentCol = statsField;
                break;
            }
        }
    }
    // 穿透配置

    // 左侧分组项
    const rowHeaders = Info.rowHeaders;
    var rowData = {},colData,title = "";
    for(var i=0;i<rowHeaders.length;i++){
        var header = rowHeaders[i];
        var key = header.aliasTableName + "_" + header.name;
        rowData[key] = currentRow[key][1];
        title = title + (currentRow[key][0] == undefined ? "" :  currentRow[key][0] ) + "_";
    }
    var thisKey = item.columnId;

    if(!!Data.colData && Data.colData.length>0){//有交叉项
        let colIndex =  item.columnId.substring(4,item.columnId.indexOf('_',4));
        //当前的交叉
        var currentCross = Data.colData[colIndex] ? Data.colData[colIndex]:['',''];
        colData = currentCross[1];
        title = title + (currentCross[0] == undefined ? "" :  currentCross[0] ) + "_";
    }
    title = title + currentCol.aliasDisplay;
    if(currentCol.calcType == "sum"){
        title = title + "(求和)";
    }else if(currentCol.calcType == "count"){
        title = title + "(计数)";
    }else if(currentCol.calcType == "avg"){
        title = title + "(平均值)";
    }else if(currentCol.calcType == "max"){
        title = title + "(最大值)";
    }else if(currentCol.calcType == "min"){
        title = title + "(最小值)";
    }
    title = title + ":" + (currentRow[thisKey][0]?currentRow[thisKey][0]:'');
    if(!!title)title = escapeStringToHTML(title,true,false);
    const statisticItem = currentCol;
    let params = {
        designId,
        rowData,
        colData,
        userConditions,
        statisticItem
    }
    let url = _ctxPath + "/report4Result.do?method=reportPenetrate&rptDesignId="+designId+"&statisticItem="+encodeURI(JSON.stringify(currentCol))
    // windows绑定
    let statisticsPopData = {url,title,params};
    window.top.getStatisticsPopParams = () => statisticsPopData;
    return statisticsPopData;
}
